@@ -8,7 +8,7 @@ use CGI();
 BEGIN {
     use Exporter ();
     use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
-    $VERSION     = '0.15';
+    $VERSION     = '0.16';
     @ISA         = qw(Exporter);
     #Give a hoot don't pollute, do not export more than needed by default
     @EXPORT      = qw();
@@ -364,7 +364,12 @@ intercept messages sent to the error handler. Very useful if you are running in
 
 =item B<< hiddenFilter => "[SELECT_COL_ALIAS] like 'foo' AND .." >>
 
-default filters to include on a fresh loaded OptimalQuery. The value is translated to an SQL where clause using the grammar described in the I<FILTER GRAMMAR> section. Both filter options can also be set by CGI param.  Example: 
+=item B<< forceFilter => "[SELECT_COL_ALIAS] like 'foo' AND .." >>
+
+Filters add SQL to the where clause. If a CGI param called 'filter' or 'hiddenFilter' are provided, the CGI param value is used instead. The value of a forceFilter cannot be overridden. Users can manipulate a filter using the filter dialog tool. The hiddenFilter can only be manipulated using the GET param. Filters allow developers and end users to add SQL to the where clause. The filter grammar is described in the I<FILTER GRAMMAR> section.
+
+For example:
+
   <a href=/Search?filter=".escape_uri("[NAME] like 'foo'")
 
 =item B<< module => { OverloadModuleLabel => PerlModuleName, .. } >>
@@ -436,6 +441,19 @@ An interactive query displays a pager mechanism when the result set is larger th
 
 The default rows_page a user has when initially loading InteractiveQuery. Can also be set as a CGI param.
 
+=item B<< mutateRecord => sub { } >>
+
+  $$schema{mutateRecord} = sub {
+    my $rec = shift;
+
+    # add html links to the person record 
+    # if user selected the NAME field
+    if (exists $$rec{NAME}) {
+      $$rec{NAME} = "<A HREF=/PersonRecord?id=$$rec{ID}>".
+        CGI::escapeHTML($$rec{NAME})."</A>";
+    }
+  };
+
 =item B<< savedSearchUserID => $user_id >>
 
 InteractiveQuery can optionally save searches to a database so users can revisit them latter. To do this, saved searches are tied to a unique user id. See the "Saved Searches" section for more information on this topic.
@@ -444,6 +462,14 @@ InteractiveQuery can optionally save searches to a database so users can revisit
 
 If savedSearchUserID, is defined, setting savedSearchAlerts to 1 will enhance dialogs to optionally allow users to configure saved searches to alert them when records are added, removed, or are present. See the "Saved Search Alerts" section for additional details on configuring this feature.
 
+=item B<< savedSearchAlertMaxRecs => 1000 >>
+
+Specify maximum record UIDs to store in saved search alert previously seen uid history. The previously stored list of UIDs is used to determine if a new record has appeared. Don't set this too high - the pks are stored in the saved search table. The default settings allow about 14k of data per saved search. (15 char key size * 1000 savedSearchAlertMaxRecs) / 1024 = 14k.
+
+=item B<< savedSearchAlertEmailCharLimit => 500000 >>
+
+Maximum character limit of saved saerch alert email. If limit is reached, a message appears instructing the user reduce the size by hiding fields or modify filters. The default value is 500000 (~ .5MB)
+
 =item B<< sort => "[COLALIAS1] DESC, [COLALIAS2]" >>
 
 Default sort to show user when loading OptimalQuery for the first time. See the "SORT GRAMMAR" section for more information. Sort can also be specified as a CGI param.
@@ -478,21 +504,6 @@ Options for an InteractiveQuery can be set by defining the following HASHREF in
 
 =over
 
-=item B<< mutateRecord => sub { } >>
-
-A mutate mutateRecord option can also be set in the options hash. This is useful if you only want to apply the mutation for a particular output module.
-
-  mutateRecord => sub {
-    my $rec = shift;
-
-    # add html links to the person record 
-    # if user selected the NAME field
-    if (exists $$rec{NAME}) {
-      $$rec{NAME} = "<A HREF=/PersonRecord?id=$$rec{ID}>".
-        CGI::escapeHTML($$rec{NAME})."</A>";
-    }
-  }
-
 =item B<< disable_select => 1 >>
 
 =item B<< disable_filter => 1 >>
@@ -648,7 +659,7 @@ Saved Searches are stored in a table in the database pointed to by the database
     user_title VARCHAR2(1000) NOT NULL,
     params CLOB,
     -- 0:disabled, 1:added, 2:removed, 4:present
-    alert_mask NUMBER NOT NULL DEFAULT 0,
+    alert_mask NUMBER DEFAULT 0 NOT NULL,
     alert_interval_min NUMBER,
     alert_dow VARCHAR2(7),
     alert_start_hour NUMBER,
