@@ -52,6 +52,9 @@ sub new {
   return $sth;
 }
 
+sub get_lo_rec { $_[0]{limit}[0] }
+sub get_hi_rec { $_[0]{limit}[1] }
+
 # execute statement
 # notice that we can't execute other child cursors
 # because their bind params are dependant on
@@ -62,14 +65,20 @@ sub execute {
   $$sth{oq}{error_handler}->("DEBUG: \$sth->execute(\n".Dumper(\%args).")\n") if $$sth{oq}{debug};
 
   my $count = $sth->count();
+
   $args{limit} ||= [];
   if ($count > 0) {
-    my $lo_rec = $args{limit}[0] || 1;
-    my $hi_rec = $args{limit}[1] || $count;
-    $hi_rec = $count if $args{limit}[1] eq '*';
-    $sth->add_limit_sql($lo_rec, $hi_rec);
+    $args{limit}[0] ||= 1;
+    $args{limit}[1] ||= $count;
+    $args{limit}[1] = $count if $args{limit}[1] eq '*';
+    $sth->add_limit_sql($args{limit}[0],$args{limit}[1]);
+  } else {
+    $args{limit} = [0,0];
   }
 
+  # preserve limit in object
+  $$sth{limit} = $args{limit};
+
   my $c = $sth->{cursors}->[0];
 
   # if clobs have been selected, find & set LongReadLen
@@ -379,7 +388,7 @@ sub create_where {
   my $sth = shift;
 
   $$sth{oq}{error_handler}->("DEBUG: \$sth->create_where()\n") if $$sth{oq}{debug};
-  return undef if $sth->{'filter'} eq '' && $sth->{'hiddenFilter'} eq '';
+  return undef if $sth->{'filter'} eq '' && $sth->{'hiddenFilter'} eq '' && $sth->{'forceFilter'} eq '';
 
   # this sub glues together a parsed expression
   # basically is glues statements that look like:
@@ -439,7 +448,9 @@ sub create_where {
     },
 
     'quotedString' => sub {
-      $_ = $_[2]; (s/^\'// && s/\'$//) || (s/^\"// && s/\"$//); $_;
+      my ($v) = $_[2];
+      ($v=~s/^\'// && $v=~s/\'$//) || ($v=~s/^\"// && $v=~s/\"$//);
+      return $v;
     },
 
     'exp' => sub {
@@ -606,6 +617,9 @@ sub create_where {
           } elsif ($token[1]{sql} eq '!=') {
             $token[1]{sql} = '=';
             $preSql .= 'NOT ';
+          } elsif ($token[1]{sql} eq 'is null' ) {
+            $token[1]{sql} = 'is not null';
+            $preSql .= 'NOT ';
           }
           $preSql  .= 'EXISTS ( SELECT 1 FROM '.$fromSql.' WHERE ('.$corelatedJoin.') AND ';
           $postSql .= ')';
@@ -681,6 +695,16 @@ sub create_where {
     push @{ $c->{where_binds} }, @{ $$hiddenFilter{binds} };
   }
 
+  # add system filter parts to cursor's where parts
+  if ($sth->{'forceFilter'} ne '') {
+    my $forceFilter = $$sth{oq}->parse($DBIx::OptimalQuery::filterGrammar, $sth->{'forceFilter'}, \%translations)
+      or die "could not parse forceFilter: ".$sth->{'forceFilter'};
+    push @{ $c->{where_deps} }, @{ $$forceFilter{deps} };
+    $c->{where_sql}  = '('.$c->{where_sql}.') AND ' if $c->{where_sql} ne '';
+    $c->{where_sql}  .= '('.$$forceFilter{sql}.')';
+    push @{ $c->{where_binds} }, @{ $$forceFilter{binds} };
+  }
+
   return undef;
 }
 
